package ch.ethz.karto.map3d;

import java.nio.ByteBuffer;
import javax.media.opengl.GL;

/**
 * 2D single byte texture.
 * @author Bernhard Jenny, Institute of Cartography, ETH Zurich.
 */
public class Map3DTextureByte {

    private int[] textureName = new int[1];

    public Map3DTextureByte(GL gl, ByteBuffer buffer, int width, int height) {
        load(gl, buffer, width, height);
    }

    /**
     * Passes the texture buffer to OpenGL.
     * @param gl
     */
    private void load(GL gl, ByteBuffer buffer, int width, int height) {

        buffer.rewind();

        // generate a new texture name
        gl.glGenTextures(1, textureName, 0);

        // bind and load the texture
        gl.glBindTexture(GL.GL_TEXTURE_2D, textureName[0]);

        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, GL.GL_CLAMP_TO_EDGE);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_CLAMP_TO_EDGE);
        gl.glPixelStorei(GL.GL_UNPACK_ALIGNMENT, 1);
        gl.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_LUMINANCE, width, height, 0,
                GL.GL_LUMINANCE, GL.GL_UNSIGNED_BYTE, buffer);
    }

    public void release(GL gl) {
        gl.glDeleteTextures(textureName.length, textureName, 0);
    }

    public int getTextureName() {
        return textureName[0];
    }
    
}
